@@ -55,7 +55,7 @@ static int ebayLogin(auctionInfo *aip, int interval);
 static char *getIdInternal(char *s, size_t len);
 static int getInfoTiming(auctionInfo *aip, time_t *timeToFirstByte);
 static int getQuantity(int want, int available);
-static int makeBidError(const char *pagename, auctionInfo *aip);
+static int makeBidError(const pageInfo_t *pageInfo, auctionInfo *aip);
 static int match(memBuf_t *mp, const char *str);
 static int parseBid(memBuf_t *mp, auctionInfo *aip);
 static int preBid(auctionInfo *aip);
@@ -109,6 +109,7 @@ getPageInfo(memBuf_t *mp)
 	int needMore = 3;
 
 	log(("getPageInfo():\n"));
+	memReset(mp);
 	while (needMore && (line = getTag(mp))) {
 		char *tmp;
 
@@ -135,6 +136,7 @@ getPageInfo(memBuf_t *mp)
 		}
 	}
 	log(("getPageInfo(): pageName = %s, pageId = %s, srcId = %s\n", nullStr(p.pageName), nullStr(p.pageId), nullStr(p.srcId)));
+	memReset(mp);
 	if (needMore == 3)
 		return NULL;
 	pp = (pageInfo_t *)myMalloc(sizeof(pageInfo_t));
@@ -233,7 +235,7 @@ getInfoTiming(auctionInfo *aip, time_t *timeToFirstByte)
 			freeMembuf(mp);
 			return httpError(aip);
 		}
-		ret = parseBidHistory(mp, aip, start, timeToFirstByte);
+		ret = parseBidHistory(mp, aip, start, timeToFirstByte, 0);
 		freeMembuf(mp);
 		if (i == 0 && ret == 1 && aip->auctionError == ae_mustsignin) {
 			if (ebayLogin(aip, -1))
@@ -320,14 +322,14 @@ preBid(auctionInfo *aip)
 		}
 	}
 	if (found < 7) {
-		char *pagename;
+		pageInfo_t *pageInfo = getPageInfo(mp);
 
-		mp->readptr = mp->memory;
-		pagename = getPageName(mp);
-		if ((ret = makeBidError(pagename, aip)) < 0) {
+		ret = makeBidError(pageInfo, aip);
+		if (ret < 0) {
 			ret = auctionError(aip, ae_bidkey, NULL);
 			bugReport("preBid", __FILE__, __LINE__, aip, mp, "cannot find bid key, uiid or password, found = %d", found);
 		}
+		freePageInfo(pageInfo);
 	}
 	freeMembuf(mp);
 	return ret;
@@ -462,12 +464,26 @@ acceptBid(const char *pagename, auctionInfo *aip)
  * Returns -1 if page not recognized, 0 if bid accepted, 1 if bid not accepted.
  */
 static int
-makeBidError(const char *pagename, auctionInfo *aip)
+makeBidError(const pageInfo_t *pageInfo, auctionInfo *aip)
 {
 	static const char MAKEBIDERROR[] = "MakeBidError";
+	const char *pagename = pageInfo->pageName;
 
-	if (!pagename ||
-	    strncmp(pagename, MAKEBIDERROR, sizeof(MAKEBIDERROR) - 1))
+	if (!pagename) {
+		const char *srcId = pageInfo->srcId;
+
+		if (srcId && !strcmp(srcId, "ViewItem"))
+			return aip->bidResult = auctionError(aip, ae_ended, NULL);
+		else
+			return -1;
+	}
+	if (!strcmp(pagename, "PageSignIn"))
+		return aip->bidResult = auctionError(aip, ae_mustsignin, NULL);
+	if (!strncmp(pagename, "BidManager", 10) ||
+	    !strncmp(pagename, "BidAssistant", 12))
+		return aip->bidResult = auctionError(aip, ae_bidassistant, NULL);
+
+	if (strncmp(pagename, MAKEBIDERROR, sizeof(MAKEBIDERROR) - 1))
 		return -1;
 	pagename += sizeof(MAKEBIDERROR) - 1;
 	if (!*pagename ||
@@ -511,17 +527,21 @@ parseBid(memBuf_t *mp, auctionInfo *aip)
 	 * example AcceptBid_HighBidder_rebid (you were already the high
 	 * bidder and placed another bid).
 	 */
-	char *pagename = getPageName(mp);
+	pageInfo_t *pageInfo = getPageInfo(mp);
 	int ret;
 
 	aip->bidResult = -1;
-	log(("parseBid(): pagename = %s\n", pagename));
-	if ((ret = acceptBid(pagename, aip)) >= 0 ||
-	    (ret = makeBidError(pagename, aip)) >= 0)
-		return ret;
-	bugReport("parseBid", __FILE__, __LINE__, aip, mp, "unknown pagename");
-	printLog(stdout, "Cannot determine result of bid\n");
-	return 0;	/* prevent another bid */
+	log(("parseBid(): pagename = %s\n", pageInfo->pageName));
+	if ((ret = acceptBid(pageInfo->pageName, aip)) >= 0 ||
+	    (ret = makeBidError(pageInfo, aip)) >= 0) {
+		;
+	} else {
+		bugReport("parseBid", __FILE__, __LINE__, aip, mp, "unknown pagename");
+		printLog(stdout, "Cannot determine result of bid\n");
+		ret = 0;	/* prevent another bid */
+	}
+	freePageInfo(pageInfo);
+	return ret;
 } /* parseBid() */
 
 static const char BID_URL[] = "http://%s/ws/eBayISAPI.dll?MfcISAPICommand=MakeBid&BIN_button=Confirm%%20Bid&item=%s&key=%s&maxbid=%s&quant=%s&user=%s&pass=%s&uiid=%s&javascriptenabled=0&mode=1";
@@ -677,6 +697,9 @@ watch(auctionInfo *aip)
 				if (!preBid(aip) ||
 				    aip->auctionError == ae_bidkey)
 					break;
+				if (aip->auctionError == ae_mustsignin &&
+				    ebayLogin(aip, -1))
+					break;
 			}
 			if (aip->auctionError != ae_none &&
 			    aip->auctionError != ae_highbidder) {
@@ -773,10 +796,17 @@ snipeAuction(auctionInfo *aip)
 
 	if (aip->auctionError != ae_highbidder) {
 		printLog(stdout, "\nAuction %s: Bidding...\n", aip->auction);
-		if (bid(aip)) {
-			/* failed bid */
-			printAuctionError(aip, stderr);
-			return 0;
+		for (;;) {
+			if (bid(aip)) {
+				/* failed bid */
+				if (aip->auctionError == ae_mustsignin) {
+					if (!ebayLogin(aip, -1))
+						continue;
+				}
+				printAuctionError(aip, stderr);
+				return 0;
+			}
+			break;
 		}
 	}
 
@@ -818,7 +848,7 @@ snipeAuction(auctionInfo *aip)
 	return won;
 }
 
-#define MAX_TDS 6
+#define MAX_TDS 7
 
 /*
  * On first call, use printNewline to 0.  On subsequent calls, use return
@@ -827,7 +857,7 @@ snipeAuction(auctionInfo *aip)
 static int
 printMyItemsRow(char **row, int printNewline)
 {
-	const char *myitems_description[7][MAX_TDS] = {
+	const char *myitems_description[MAX_TDS + 1][MAX_TDS] = {
 		{0, 0, 0, 0, 0, 0},
 		{"Note:\t\t%s\n", 0, 0, 0, 0, 0},
 		{0, 0, 0, 0, 0, 0},
@@ -835,7 +865,9 @@ printMyItemsRow(char **row, int printNewline)
 		{0, 0, "Description:\t%s\n", 0, 0, 0},
 		{0, 0, 0, 0, 0, 0},
 		{0, "Price:\t\t%s\n", "Shipping:\t%s\n", "Bids:\t\t%s\n",
-		 "Seller:\t\t%s\n", "Time:\t\t%s\n"}
+		 "Seller:\t\t%s\n", "Time:\t\t%s\n", 0},
+		{0, "Price:\t\t%s\n", "Shipping:\t%s\n", "Bids:\t\t%s\n",
+		 "Seller:\t\t%s\n", 0, "Time:\t\t%s\n"}
 	};
 	int nColumns = numColumns(row);
 	int column = 0;
@@ -936,7 +968,6 @@ printMyItems(void)
 	return 0;
 }
 
-#if DEBUG
 /* secret option - test parser */
 void
 testParser(int flag)
@@ -954,7 +985,7 @@ testParser(int flag)
 			printf("\"%s\"\n", line);
 
 		/* pagename? */
-		mp->readptr = mp->memory;
+		memReset(mp);
 		if ((line = getPageName(mp)))
 			printf("\nPAGENAME is \"%s\"\n", line);
 		else
@@ -966,7 +997,7 @@ testParser(int flag)
 		/* run through bid history parser */
 		auctionInfo *aip = newAuctionInfo("1", "2");
 		time_t start = time(NULL), end;
-		int ret = parseBidHistory(mp, aip, start, &end);
+		int ret = parseBidHistory(mp, aip, start, &end, 1);
 
 		printf("ret = %d\n", ret);
 		printAuctionError(aip, stdout);
@@ -1022,4 +1053,3 @@ testParser(int flag)
 	    }
 	}
 }
-#endif
